/*
 * SimpleReceiver.cpp
 *
 * Demonstrates receiving NEC IR codes with IRremote
 *
 *  This file is part of Arduino-IRremote https://github.com/Arduino-IRremote/Arduino-IRremote.
 *
 * CHANGES MADE BY ME FEB. 6, 2024 to the Simple Receiver example in IRremote library:   
 *     Previous Lines 49-50: Changed NEC to SAMSUNG.   (Line numbers are before changes for Talkie.)
 *     Previous Line  80:    Changed Serial.begin to 9600.
 *     Previous Lines 106-113: Changed to not display data for UNKNOWN protocol.
 *     Previous Lines 126-133: Changed/Added: display a message if key 1 or 2 is pressed.
 *
 * MADE ADDITIONAL CHANGES TO INCLUDE TALKIE AND SERVO LIBRARIES AND STATEMENTS, FEB. 2024.
 *
 ************************************************************************************
 * MIT License
 *
 * Copyright (c) 2020-2023 Armin Joachimsmeyer
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is furnished
 * to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 * OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 ************************************************************************************
 */

/*
 * Specify which protocol(s) should be used for decoding.
 * If no protocol is defined, all protocols (except Bang&Olufsen) are active.
 * This must be done before the #include <IRremote.hpp>
 */
//#define DECODE_DENON        // Includes Sharp
//#define DECODE_JVC
//#define DECODE_KASEIKYO
//#define DECODE_PANASONIC    // alias for DECODE_KASEIKYO
//#define DECODE_LG
//#define DECODE_NEC          // Includes Apple and Onkyo
#define DECODE_SAMSUNG
//#define DECODE_SONY
//#define DECODE_RC5
//#define DECODE_RC6

//#define DECODE_BOSEWAVE
//#define DECODE_LEGO_PF
//#define DECODE_MAGIQUEST
//#define DECODE_WHYNTER
//#define DECODE_FAST

//#define DECODE_DISTANCE_WIDTH // Universal decoder for pulse distance width protocols
//#define DECODE_HASH         // special decoder for all protocols

//#define DECODE_BEO          // This protocol must always be enabled manually, i.e. it is NOT enabled if no protocol is defined. It prevents decoding of SONY!

//#define DEBUG               // Activate this for lots of lovely debug output from the decoders.

//#define RAW_BUFFER_LENGTH  180  // Default is 112 if DECODE_MAGIQUEST is enabled, otherwise 100.

#include <Arduino.h>

#include "Talkie.h"            //   ******* Added for Talkie library
#include "Vocab_US_TI99.h"     //   ******* Added for Talkie
Talkie voice;                  //   ******* Added for Talkie

/*
 * This include defines the actual pin number for pins like IR_RECEIVE_PIN, IR_SEND_PIN for many different boards and architectures
 */
#include "PinDefinitionsAndMore.h"

#include <IRremote.hpp> // include the IRremote library

#include <Servo.h>
Servo myservo;        // creates servo object to control a servo

void setup() {
    Serial.begin(9600);
    // Just to know which program is running on my Arduino
    Serial.println(F("START " __FILE__ " from " __DATE__ "\r\nUsing library version " VERSION_IRREMOTE));

    // Start the receiver and if not 3. parameter specified, take LED_BUILTIN pin from the internal boards definition as default feedback LED
    IrReceiver.begin(IR_RECEIVE_PIN, ENABLE_LED_FEEDBACK);

    Serial.print(F("Ready to receive IR signals of protocols: "));
    printActiveIRProtocols(&Serial);
    Serial.println(F("at pin " STR(IR_RECEIVE_PIN)));

}

void loop() {
    /*
     * Check if received data is available and if yes, try to decode it.
     * Decoded result is in the IrReceiver.decodedIRData structure.
     *
     * E.g. command is in IrReceiver.decodedIRData.command
     * address is in command is in IrReceiver.decodedIRData.address
     * and up to 32 bit raw data in IrReceiver.decodedIRData.decodedRawData
     */
    if (IrReceiver.decode()) {

        /*
         * Print a short summary of received data
         */
         if (IrReceiver.decodedIRData.protocol != UNKNOWN) {            // added Feb 6/24
        IrReceiver.printIRResultShort(&Serial);
        //IrReceiver.printIRSendUsage(&Serial);                          ) changed into comments
        //if (IrReceiver.decodedIRData.protocol == UNKNOWN) {            ) changed
           // Serial.println(F("Received noise or an unknown (or not yet enabled) protocol")); ) changed
           // // We have an unknown protocol here, print more info       ) changed
           // IrReceiver.printIRResultRawFormatted(&Serial, true);       ) changed
        //}                                                              ) changed
        Serial.println();
         }
        /*
         * !!!Important!!! Enable receiving of the next value,
         * since receiving has stopped after the end of the current received data packet.
         */
        IrReceiver.resume(); // Enable receiving of the next value
            IrReceiver.stop();     //   ******* Added for Talkie
        /*
         * Finally, check the received data and perform actions according to the received command
         */
        if (IrReceiver.decodedIRData.command == 0x4) {
            Serial.print(F("key 1 was pressed"));          //  added Feb 6/24
            Serial.println();                              //  added
                myservo.detach();     // initialize the timer for Servo.
                myservo.attach(9);
                myservo.write(90);    // 90 degrees (Opens the lips.)
                delay(300);           // wait for servo move, before talking. (It needs 300 msec after 90 degrees.)
              voice.say(spt_HELLO);
                 myservo.detach();    // initialize the timer for Servo.
                 myservo.attach(9);  
                 myservo.write(45);   // 45 degrees (Closes the lips partway.)
                 delay(300);          // wait for servo move, before talking. (300 msec is needed again here for proper move, but not in other phrases.)
              voice.say(spt_THERE);
                 myservo.detach();    // initialize the timer for Servo.
                 myservo.attach(9);  
                 myservo.write(0);    // zero degrees (Closes the lips.)
                 delay(200);          // wait for servo move, before talking. (200 msec is enough to return to zero degrees.)
        } else if (IrReceiver.decodedIRData.command == 0x5) {
            Serial.print(F("key 2 was pressed"));          //  added Feb 6/24
            Serial.println();                              //  added
                myservo.detach();     // initialize the timer for Servo.
                myservo.attach(9);
                myservo.write(45);    // 45 degrees (Opens the lips halfway.)
                delay(200);           // wait for servo move, before talking. (200 msec is enough after 45 degrees.)
              voice.say(spt_GOOD);
              voice.say(spt_TO);
              voice.say(spt_SEE);
              voice.say(spt_YOU);
                myservo.detach();     // initialize the timer for Servo.
                myservo.attach(9);  
                myservo.write(0);     // zero degrees (Closes the lips.)
                delay(200);           // wait for servo move, before talking.
        }else if (IrReceiver.decodedIRData.command == 0x6) {
            Serial.print(F("key 3 was pressed")); 
            Serial.println(); 
                myservo.detach();
                myservo.attach(9);
                myservo.write(90);    // 90 degrees (Opens the lips)
                delay(300);
              voice.say(spt_HOW);
                myservo.detach();
                myservo.attach(9);
                myservo.write(0);     // zero degrees (Closes the lips)
                delay(200);
                myservo.detach();
                myservo.attach(9);
                myservo.write(90);    // 90 degrees (Opens the lips)
                delay(300);
              voice.say(spt_ARE);
                myservo.detach();
                myservo.attach(9);
                myservo.write(45);    // 45 degrees (Moves the lips partway)
                delay(200);
              voice.say(spt_YOU);
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(0);    // zero degrees (Closes the lips.)
                 delay(200); 
        }else if (IrReceiver.decodedIRData.command == 0x8) {
            Serial.print(F("key 4 was pressed"));
            Serial.println(); 
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(45);   // 45 degrees (Moves the lips partway.)
                 delay(200); 
              voice.say(spt_I);
              voice.say(spt_AM);
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(0);    // zero degrees (Closes the lips.)
                 delay(200); 
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(45);   // 45 degrees (Moves the lips partway.)
                 delay(200); 
              voice.say(spt_IN);
              voice.say(spt_GOOD);
              voice.say(spt_SHAPE);
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(0);    // zero degrees (Closes the lips.)
                 delay(200); 
        }else if (IrReceiver.decodedIRData.command == 0x9) {
            Serial.print(F("key 5 was pressed")); 
            Serial.println();
                myservo.detach();
                myservo.attach(9);
                myservo.write(90);    // 90 degrees (Opens the lips)
                delay(300);
              voice.say(spt_WHAT);
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(45);   // 45 degrees (Moves the lips partway.)
                 delay(200); 
              voice.say(spt_IS);
              voice.say(spt_UP);
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(0);    // zero degrees (Closes the lips.)
                 delay(200); 
        }else if (IrReceiver.decodedIRData.command == 0xA) {
            Serial.print(F("key 6 was pressed"));
            Serial.println(); 
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(45);   // 45 degrees (Moves the lips partway.)
                 delay(200); 
              voice.say(spt_TELL);
              voice.say(spt_ME);
              voice.say(spt_MORE);
              voice.say(spt_ABOUT);
                myservo.detach();
                myservo.attach(9);
                myservo.write(90);    // 90 degrees (Opens the lips)
                delay(300);
              voice.say(spt_THAT);
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(0);    // zero degrees (Closes the lips.)
                 delay(200); 
        }else if (IrReceiver.decodedIRData.command == 0xC) {
            Serial.print(F("key 7 was pressed")); 
            Serial.println(); 
                myservo.detach();
                myservo.attach(9);
                myservo.write(90);    // 90 degrees (Opens the lips)
                delay(300);
              voice.say(spt_THAT);
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(45);   // 45 degrees (Moves the lips partway.)
                 delay(200); 
              voice.say(spt_IS);
              voice.say(spt_GOOD);
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(0);    // zero degrees (Closes the lips.)
                 delay(200); 
        }else if (IrReceiver.decodedIRData.command == 0xD) {
            Serial.print(F("key 8 was pressed")); 
            Serial.println(); 
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(45);   // 45 degrees (Moves the lips partway.)
                 delay(200); 
              voice.say(spt_SORRY);
              voice.say(spt_TO);
              voice.say(spt_HEAR);
                myservo.detach();
                myservo.attach(9);
                myservo.write(90);    // 90 degrees (Opens the lips)
                delay(300);
              voice.say(spt_THAT);
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(0);    // zero degrees (Closes the lips.)
                 delay(200); 
        }else if (IrReceiver.decodedIRData.command == 0xE) {
            Serial.print(F("key 9 was pressed")); 
            Serial.println();  
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(45);   // 45 degrees (Moves the lips partway.)
                 delay(200); 
              voice.say(spt_BE);
              voice.say(spt_WELL);
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(0);    // zero degrees (Closes the lips.)
                 delay(200); 
        }else if (IrReceiver.decodedIRData.command == 0x11) {
            Serial.print(F("key 0 was pressed")); 
            Serial.println(); 
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(45);   // 45 degrees (Moves the lips partway.)
                 delay(200); 
              voice.say(spt_I);
              voice.say(spt_THINK);
              voice.say(spt_THERE);
              voice.say(spt_FOUR);
              voice.say(spt_I);
                myservo.detach();
                myservo.attach(9);
                myservo.write(90);    // 90 degrees (Opens the lips)
                delay(300);
              voice.say(spt_AM);
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(0);    // zero degrees (Closes the lips.)
                 delay(200); 
            delay(500);
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(45);   // 45 degrees (Moves the lips partway.)
                 delay(200); 
              voice.say(spt_I);
              voice.say(spt_THINK);
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(0);    // zero degrees (Closes the lips.)
                 delay(200); 
        }else if (IrReceiver.decodedIRData.command == 0x68) {
            // do something else
            Serial.print(F("The round, middle key was pressed"));
            Serial.println(); 
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(45);   // 45 degrees (Moves the lips partway.)
                 delay(200); 
              voice.say(spt_SO);
              voice.say(spt_LONG);
                 myservo.detach();
                 myservo.attach(9);  
                 myservo.write(0);    // zero degrees (Closes the lips.)
                 delay(200); 
        }
            IrReceiver.start(8000);     //   ******* Added for Talkie
    }
}
